@@ -37,7 +37,7 @@ my class Backtrace is List {
 #?if parrot
         self.new(nqp::getattr(nqp::decont($e), Exception, '$!ex').backtrace, $offset);
 #?endif
-#?if jvm
+#?if !parrot
         self.new(nqp::backtrace(nqp::getattr(nqp::decont($e), Exception, '$!ex')), $offset);
 #?endif
     }
@@ -56,25 +56,29 @@ my class Backtrace is List {
             next if nqp::isnull($sub);
             my $code;
             try {
-                $code = nqp::getcodeobj($sub);
+                $code := nqp::getcodeobj($sub);
+                $code := Any unless nqp::istype($code, Mu);
             };
             my $line     = $bt[$_]<annotations><line>;
             my $file     = $bt[$_]<annotations><file>;
             next unless $line && $file;
             # now *that's* an evil hack
             next if $file eq 'src/gen/BOOTSTRAP.nqp' ||
-                    $file eq 'src\\gen\\BOOTSTRAP.nqp';
+                    $file eq 'src/gen/m-BOOTSTRAP.nqp' ||
+                    $file eq 'src\\gen\\BOOTSTRAP.nqp' ||
+                    $file eq 'src\\gen\\m-BOOTSTRAP.nqp';
             last if $file eq 'src/stage2/gen/NQPHLL.nqp' ||
                     $file eq 'src\\stage2\\gen\\NQPHLL.nqp' ||
                     $file eq 'gen/parrot/stage2/NQPHLL.nqp' ||
                     $file eq 'gen\\parrot\\stage2\\NQPHLL.nqp' ||
                     $file eq 'gen/jvm/stage2/NQPHLL.nqp' ||
-                    $file eq 'gen\\jvm\\stage2\\NQPHLL.nqp';
-                    # XXX extend for moar
+                    $file eq 'gen\\jvm\\stage2\\NQPHLL.nqp' ||
+                    $file eq 'gen/moar/stage2/NQPHLL.nqp' ||
+                    $file eq 'gen\\moar\\stage2\\NQPHLL.nqp';
             my $subname  = nqp::p6box_s(nqp::getcodename($sub));
             $subname = '<anon>' if $subname.substr(0, 6) eq '_block';
             $new.push: Backtrace::Frame.new(
-                :$line,
+                :line($line.Int),
                 :$file,
                 :$subname,
                 :$code,
@@ -175,3 +179,5 @@ my class Backtrace is List {
         self.grep({ !.is-hidden && (.is-routine || !.is-setting )}).join
     }
 }
+
+# vim: ft=perl6 expandtab sw=4
