@@ -204,6 +204,11 @@ my class Complex is Cool does Numeric {
         Complex.new( self.re.truncate, self.im.truncate );
     }
 
+    method narrow(Complex:D:) {
+        $!im == 0e0
+            ?? $!re.narrow
+            !! self;
+    }
 }
 
 multi sub prefix:<->(Complex:D \a) returns Complex:D {
@@ -422,4 +427,4 @@ multi postfix:<i>(Cool      \a) returns Complex:D { a.Numeric * Complex.new(0e0,
 
 constant i = Complex.new(0e0, 1e0);
 
-# vim: ft=perl6
+# vim: ft=perl6 expandtab sw=4
