@@ -1,9 +1,6 @@
 my class Cursor does NQPCursorRole {
-    has $!ast; # Need it to survive re-creations of the match object.
-    
-    # Some bits to support <prior>
-    my $last_match;
-    
+    has $!made; # Need it to survive re-creations of the match object.
+
     method MATCH() {
         my $match := nqp::getattr(self, Cursor, '$!match');
         return $match if nqp::istype($match, Match) && nqp::isconcrete($match);
@@ -11,7 +8,7 @@ my class Cursor does NQPCursorRole {
         nqp::bindattr($match, Match, '$!orig', nqp::findmethod(self, 'orig')(self));
         nqp::bindattr_i($match, Match, '$!from', nqp::getattr_i(self, Cursor, '$!from'));
         nqp::bindattr_i($match, Match, '$!to', nqp::getattr_i(self, Cursor, '$!pos'));
-        nqp::bindattr($match, Match, '$!ast', nqp::getattr(self, Cursor, '$!ast'));
+        nqp::bindattr($match, Match, '$!made', nqp::getattr(self, Cursor, '$!made'));
         nqp::bindattr($match, Match, '$!CURSOR', self);
         my Mu $list := nqp::list();
         my Mu $hash := nqp::hash();
@@ -26,8 +23,11 @@ my class Cursor does NQPCursorRole {
                     nqp::bindattr_i($match, Match, $key, $value.from);
                 }
                 else {
-                    $value := nqp::p6list($value, List, Mu)
-                        if nqp::islist($value);
+                    $value := nqp::islist($value)
+                        ?? nqp::p6list($value, Array, Mu)
+                        !! nqp::istype($value, Match)
+                            ?? $value
+                            !! [$value];
                     nqp::iscclass(nqp::const::CCLASS_NUMERIC, $key, 0)
                       ?? nqp::bindpos($list, $key, $value)
                       !! nqp::bindkey($hash, $key, $value);
@@ -41,10 +41,7 @@ my class Cursor does NQPCursorRole {
     }
 
     method MATCH_SAVE() {
-        return Nil if nqp::getattr_i(self, Cursor, '$!pos') < 0;
-        my $match := self.MATCH();
-        $last_match := $match if $match;
-        $match;
+        nqp::getattr_i(self, Cursor, '$!pos') < 0 ?? Nil !! self.MATCH()
     }
 
     # INTERPOLATE will iterate over the string $tgt beginning at position 0.
@@ -82,8 +79,8 @@ my class Cursor does NQPCursorRole {
                         if $a {
                             # We are in a regex assertion, the strings we get will be treated as
                             # regex rules.
-                            my $rx := eval( $i  ?? "my \$x = anon regex \{:i ^$topic \}"
-                                                !! "my \$x = anon regex \{ ^$topic \}" );
+                            return $cur.'!cursor_start_cur'() if $topic ~~ Associative;
+                            my $rx := MAKE_REGEX($topic, :$i);
                             my Mu $nfas := nqp::findmethod($rx, 'NFA')($rx);
                             $nfa.mergesubstates($start, 0, $fate, $nfas, Mu);
                         }
@@ -127,8 +124,8 @@ my class Cursor does NQPCursorRole {
                 if $a {
                     # We are in a regex assertion, the strings we get will be treated as
                     # regex rules.
-                    my $rx := eval( $i  ?? "my \$x = anon regex \{:i ^$topic \}"
-                                        !! "my \$x = anon regex \{ ^$topic \}" );
+                    return $cur.'!cursor_start_cur'() if $topic ~~ Associative;
+                    my $rx := MAKE_REGEX($topic, :$i);
                     $match := (nqp::substr($tgt, $pos, $eos - $pos) ~~ $rx).Str;
                     $len   := nqp::chars( $match );
                 }
@@ -170,18 +167,16 @@ my class Cursor does NQPCursorRole {
         $lang_cursor."$name"(); 
     }
     
+    method INDRULE($rule, |c) {
+        $rule(self, |c)
+    }
+    
     method RECURSE() {
         nqp::getlexdyn('$?REGEX')(self)
     }
-    
-    method prior() {
-        nqp::isconcrete($last_match) ??
-            self."!LITERAL"(nqp::unbox_s(~$last_match)) !!
-            self."!cursor_start_cur"()
-    }
 }
 
-sub MAKE_REGEX($arg) {
+sub MAKE_REGEX($arg, :$i) {
     my role CachedCompiledRegex {
         has $.regex;
     }
@@ -192,10 +187,21 @@ sub MAKE_REGEX($arg) {
         $arg.regex
     }
     else {
-        my $rx := eval("my \$x = anon regex \{ $arg \}");
+        my Mu $chars := nqp::split('', $arg);
+        my $k := 0;
+        my $iter := nqp::iterator($chars);
+        while $iter {
+            my $ord := nqp::ord( nqp::shift($iter) );
+            nqp::bindpos($chars, $k, "\\c[$ord]") if $ord <= 32;
+            $k := $k + 1;
+        }
+        my $arg2 := nqp::join('', $chars);
+        my $rx := $i ?? EVAL("anon regex \{ :i ^$arg2\}") !! EVAL("anon regex \{ ^$arg2\}");
         $arg does CachedCompiledRegex($rx);
         $rx
     }
 }
 
 
+
+# vim: ft=perl6 expandtab sw=4
