@@ -1,11 +1,12 @@
 my role IO::Socket does IO {
     has $!PIO;
-    has $!buffer =
+    # JVM and Parrot have a buffer here; Moar does enough buffering of its own
+    # and gets it much more correct when bytes cross boundaries, so we use its.
 #?if parrot
-        '';
+    has $!buffer = '';
 #?endif
-#?if !parrot
-        buf8.new;
+#?if jvm
+    has $!buffer = buf8.new;
 #?endif
 
     # if bin is true, will return Buf, Str otherwise
@@ -39,9 +40,9 @@ my role IO::Socket does IO {
             $rec
         }
 #?endif
-#?if !parrot
+#?if jvm
         if $!buffer.elems < $chars {
-            my $r := nqp::readfh($!PIO, nqp::decont(buf8.new), 512);
+            my $r := nqp::readfh($!PIO, nqp::decont(buf8.new), 65536);
             $!buffer ~= $r;
         }
         
@@ -66,6 +67,16 @@ my role IO::Socket does IO {
             }
             $rec;
         }
+#?endif
+#?if moar
+        if $bin {
+            nqp::readfh($!PIO, nqp::decont(buf8.new),
+                $chars == $Inf ?? 1048576 !! $chars.Int);
+        }
+        else {
+            nqp::p6box_s(nqp::readcharsfh($!PIO,
+                $chars == $Inf ?? 1048576 !! $chars.Int));
+        }
 #?endif
     }
 
@@ -139,3 +150,5 @@ my role IO::Socket does IO {
 #?endif
     }
 }
+
+# vim: ft=perl6 expandtab sw=4
