@@ -93,6 +93,8 @@ my class Int does Real { # declared in BOOTSTRAP
         elsif $x +& 0x2   { $msb += 1; }
         $msb;
     }
+
+    method narrow(Int:D:) { self }
 }
 
 multi prefix:<++>(Int:D \a is rw) {   # XXX
@@ -300,3 +302,5 @@ multi sub lsb(Int:D \i) { i.lsb }
 
 proto sub msb($) {*}
 multi sub msb(Int:D \i) { i.msb }
+
+# vim: ft=perl6 expandtab sw=4
