@@ -147,8 +147,20 @@ multi sub warn(*@msg) is hidden_from_backtrace {
     0;
 }
 
-proto sub eval($, *%) {*}
+proto sub eval($, *%) {*}  # is DEPRECATED doesn't work in settings
 multi sub eval(Str $code, :$lang = 'perl6', PseudoStash :$context) {
+    DEPRECATED("'EVAL'");
+    my $eval_ctx := nqp::getattr(nqp::decont($context // CALLER::CALLER::), PseudoStash, '$!ctx');
+    my $?FILES   := 'eval_' ~ (state $no)++;
+    my $compiler := nqp::getcomp($lang);
+    X::Eval::NoSuchLang.new(:$lang).throw
+        if nqp::isnull($compiler);
+    my $compiled := $compiler.compile($code, :outer_ctx($eval_ctx), :global(GLOBAL));
+    nqp::forceouterctx(nqp::getattr($compiled, ForeignCode, '$!do'), $eval_ctx);
+    $compiled();
+}
+proto sub EVAL($, *%) {*}
+multi sub EVAL(Str $code, :$lang = 'perl6', PseudoStash :$context) {
     my $eval_ctx := nqp::getattr(nqp::decont($context // CALLER::CALLER::), PseudoStash, '$!ctx');
     my $?FILES   := 'eval_' ~ (state $no)++;
     my $compiler := nqp::getcomp($lang);
@@ -170,20 +182,36 @@ my class Proc::Status { ... }
 
 sub run(*@args ($, *@)) {
     my $status = Proc::Status.new( :exit(255) );
+    my Mu $hash-with-containers := nqp::getattr(%*ENV, EnumMap, '$!storage');
+    my Mu $hash-without         := nqp::hash();
+    my Mu $enviter := nqp::iterator($hash-with-containers);
+    my $envelem;
+    while $enviter {
+        $envelem := nqp::shift($enviter);
+        nqp::bindkey($hash-without, nqp::iterkey_s($envelem), nqp::decont(nqp::iterval($envelem)))
+    }
+    my Mu $args-without := nqp::list();
+    for @args.eager {
+        nqp::push($args-without, nqp::decont($_));
+    }
     try {
-        my Mu $hash := nqp::getattr(%*ENV, EnumMap, '$!storage');
-        $status.status( nqp::p6box_i(
-            nqp::spawn(nqp::getattr(@args.eager, List, '$!items'), $*CWD.Str, $hash)
-        ) );
+        $status.status( nqp::p6box_i( nqp::spawn($args-without, $*CWD.Str, $hash-without) ) );
     }
     $status
 }
 
 sub shell($cmd) {
     my $status = Proc::Status.new( :exit(255) );
+    my Mu $hash-with-containers := nqp::getattr(%*ENV, EnumMap, '$!storage');
+    my Mu $hash-without         := nqp::hash();
+    my Mu $enviter := nqp::iterator($hash-with-containers);
+    my $envelem;
+    while $enviter {
+        $envelem := nqp::shift($enviter);
+        nqp::bindkey($hash-without, nqp::iterkey_s($envelem), nqp::decont(nqp::iterval($envelem)))
+    }
     try {
-        my Mu $hash := nqp::getattr(%*ENV, EnumMap, '$!storage');
-        $status.status( nqp::p6box_i(nqp::shell($cmd, $*CWD.Str, $hash)) );
+        $status.status( nqp::p6box_i(nqp::shell($cmd, $*CWD.Str, $hash-without)) );
     }
     $status
 }
@@ -207,8 +235,15 @@ sub QX($cmd) {
     $result;
 #?endif
 #?if !parrot
-    my Mu $env := nqp::getattr(%*ENV, EnumMap, '$!storage');
-    my Mu $pio := nqp::openpipe(nqp::unbox_s($cmd), $*CWD.Str, $env, '');
+    my Mu $hash-with-containers := nqp::getattr(%*ENV, EnumMap, '$!storage');
+    my Mu $hash-without         := nqp::hash();
+    my Mu $enviter := nqp::iterator($hash-with-containers);
+    my $envelem;
+    while $enviter {
+        $envelem := nqp::shift($enviter);
+        nqp::bindkey($hash-without, nqp::iterkey_s($envelem), nqp::decont(nqp::iterval($envelem)))
+    }
+    my Mu $pio := nqp::openpipe(nqp::unbox_s($cmd), $*CWD.Str, $hash-without, '');
     fail "Unable to execute '$cmd'" unless $pio;
     my $result = nqp::p6box_s(nqp::readallfh($pio));
     nqp::closefh($pio);
@@ -219,3 +254,5 @@ sub QX($cmd) {
 sub EXHAUST(|) {
     X::ControlFlow::Return.new.throw();
 }
+
+# vim: ft=perl6 expandtab sw=4
